import axios from "axios";
import Qs from "qs";
import Constant from "@/utils/constant";
import {
  getToken
} from "@/utils/tokenUtil";
import {
  uuid,
  createSign
} from "@/utils/cryptoUtil";
import {
  MessageBox,
  Message
} from "element-ui";
import {
  refreshToken,
  frontLogout
} from "@/utils/authUtil";

/**
 * axios请求方式写法
 * Content-Type: application/json(axios默认)    --->   let data = {"code":"1234","name":"yyyy"};  axios.post(`/test/testRequest`,data)
 * Content-Type: multipart/form-data    --->  let data = new FormData();  data.append('code','1234');  data.append('name','yyyy');  axios.post(`/test/testRequest`,data)
 * Content-Type: application/x-www-form-urlencoded    --->  axios.post(`/test/testRequest`,qs.stringify(data))
 */

// 是否正在刷新的标记
let isRefreshing = false;
// 重试队列，每一项将是一个待执行的函数形式
let requestQueue = [];

// 配置新建一个 axios 实例
const service = axios.create({
  // axios中请求配置有baseURL选项，表示请求URL公共部分
  baseURL: Constant.baseURL,
  // 超时
  timeout: 50000,
  // cookie跨域必备
  withCredentials: true,
  headers: {
    'Content-Type': 'application/x-www-form-urlencoded;charset=UTF-8'
  }
});

// 添加请求拦截器
service.interceptors.request.use(
  config => {
    // 每次发送请求之前判断缓存中是否存在token
    // 如果存在，则统一在http请求的header都加上token，这样后台根据token判断你的登录情况
    // 即使缓存存在token，也有可能token是过期的，所以在响应拦截器中要对返回状态进行判断
    if (getToken()) {
      config.headers[Constant.requestTokenKey] = getToken();
    }
    // 所有请求都要添加登录标识、时间戳、随机串、签名
    config.headers[Constant.equipmentKey] = "pc";
    config.headers[Constant.timestampKey] = new Date().getTime();
    config.headers[Constant.nonceKey] = uuid();

    let params; //主要用于签名计算
    if (config.method === "get" || config.method === "delete") {
      // get和delete请求参数在config.params中
      params = decodeURIComponent(config.params);
    } else {
      params = decodeURIComponent(config.data);
    }
    // Qs的解析数组部分不支持我们需要的格式，所以自己搞定
    const _ps = params.split("&");
    if (_ps) {
      params = {}
      for (let _p of _ps) {
        const _o = _p.split("=");
        if (_o) {
          params[_o[0]] = _o[1];
        }
      }
    }
    // 此处需要去除值为undefined的参数，因为后台request.getParameterMap()不会有这个参数
    for (const key in params) {
      if (params[key] === undefined) {
        delete params[key];
      }
    }
    config.headers[Constant.signatureKey] = createSign(
      config.headers[Constant.timestampKey],
      config.headers[Constant.nonceKey],
      params
    );

    return config;
  },
  error => {
    // 对请求错误做些什么
    return Promise.reject(error);
  }
);

// 添加响应拦截器
service.interceptors.response.use(
  response => {
    if (response.request.responseType === 'blob') {
      // 文件下载
      if (response.headers['content-disposition']) {
        // 返回待返还文件
        // 提取文件名
        const filename = response.headers['content-disposition'].match(/filename=(.*)/)[1]
        const blob = response.data
        const reader = new FileReader()
        // 读取Blob内容并将文件数据转换为base64，可以直接放入a标签href
        reader.readAsDataURL(blob)
        reader.onload = function (e) {
          // 转换完成，创建一个a标签用于下载
          const a = document.createElement('a')
          // decodeURI(filename) 解码文件名
          a.download = decodeURI(filename)
          a.href = e.target.result
          document.body.append(a)
          a.click()
          document.body.removeChild(a)
        }
      }
    } else {
      // 未设置状态码则默认成功状态
      const code = response.data.code || Constant.httpCode.OK;
      if (code === Constant.httpCode.OK) {
        return response
      } else if (code === Constant.httpCode.TOKEN_FAILURE) {
        // accessToken过期（4001是自定义的，需要与后端配合）
        const config = response.config;
        if (!isRefreshing) {
          isRefreshing = true;
          return refreshToken()
            .then(() => {
              // 已经刷新了token，将所有队列中的请求进行重试
              requestQueue.forEach(cb => cb());
              // 重试完清空这个队列
              requestQueue = [];
              return service(config);
            })
            .catch((error) => {
              Message.warning(`登录状态已过期...`);
              frontLogout().then(() => {
                location.href = "/";
              });
            })
            .finally(() => {
              isRefreshing = false;
            });
        } else {
          // 正在刷新token，将返回一个未执行resolve的promise
          return new Promise(resolve => {
            // 将resolve放进队列，用一个函数形式来保存，等token刷新后直接执行
            requestQueue.push(() => {
              resolve(service(config));
            });
          });
        }
      }
      return Promise.reject(response)
    }
  },
  error => {
    // 对响应错误做点什么
    if (error && error.response) {
      const code = error.response.status
      const data = error.response.data
      const msg = data.msg

      switch (code) {
        // 请求参数有问题
        case Constant.httpCode.BAD_REQUEST:
          Message.error(`请求参数有问题，${msg}`);
          break;
        // 未认证（没登录）
        case Constant.httpCode.UNAUTHORIZED:
          // 跳转登录页面，并将要浏览的页面fullPath传过去，登录成功后跳转需要访问的页面
          MessageBox.confirm("登录信息已过期，请重新登录！", "系统提示", {
            confirmButtonText: "重新登录",
            cancelButtonText: "取消",
            type: "warning"
          }).then(() => {
            frontLogout().then(() => {
              location.href = "/";
            });
          });
          break;
        // 未授权（无权限）
        case Constant.httpCode.NOT_PERMISSION:
          // 跳转到403页面(无权限)
          Message.error(`[403未授权]${msg}`);
          break;
        // 请求不存在
        case Constant.httpCode.NOT_FOUND:
          // 跳转到404页面
          Message.error(`[404没有该资源]${msg}`);
          break;
        // 其他错误，首先直接抛出错误提示，其它的自行处理
        default:
          if (error.message.indexOf("timeout") != -1) {
            Message.error("系统接口请求超时");
          } else if (error.message === "Network Error") {
            Message.error("后端接口连接异常");
          } else if (!msg && error.message.includes("Request failed with status code")) {
            Message.error(
              "系统接口" + error.message.substr(error.message.length - 3) + "异常"
            );
          } else {
            if (msg) {
              Message.error(msg);
            } else {
              Message.error("接口路径找不到");
            }
          }
      }

      return Promise.reject(data)
    }
    return Promise.reject(error);
  }
);

// 包装数据
const _axios = function (url, config) {
  if (typeof url === "string" || url instanceof String) {
    if (config) {
      config["url"] = url;
    } else {
      return service(url);
    }
  } else {
    config = url;
  }
  // 如果要自己定义的话，这里就不管了，数据格式要自己处理，另外，还要考虑签名认证的问题->这是重点
  if (config["headers"] && config["headers"]["Content-Type"]) {
    return service(config);
  }
  // 新加的isList属性，方便后台接收list数据，例如：ids
  let isList = config["isList"] ? config["isList"] : false;
  if (config["method"] == "get" || config["method"] == "delete") {
    // get和delete默认使用的是params属性，这里为了使用方便，统一使用data，将data自动转到params。
    if (config["data"]) {
      config["params"] = config["data"]
      delete config["data"];
    }
    // isList和正常提交的数据格式不一样
    if (isList) {
      config["paramsSerializer"] = params => {
        return Qs.stringify(params, {
          arrayFormat: 'repeat',
          allowDots: true
        });
      };
    } else {
      config["paramsSerializer"] = params => {
        return Qs.stringify(params, {
          arrayFormat: 'indices',
          allowDots: true
        });
      };
    }
  } else if (config["method"] == "post" || config["method"] == "put") {
    // isList和正常提交的数据格式不一样
    if (isList) {
      config["data"] = Qs.stringify(config["data"], {
        arrayFormat: 'repeat',
        allowDots: true
      });
    } else {
      config["data"] = Qs.stringify(config["data"], {
        arrayFormat: 'indices',
        allowDots: true
      });
    }
  }
  return service(config);
}

export default _axios;
