/**
 * 表单项基础类, 所有输入组件都继承item
 */

const _get = require("lodash/get");
const _set = require("lodash/set");
import { cloneDeep, isEqual,isObject } from "@/utils/commonUtil";
export default {
  inject: {
    mForm: {
      default: null
    }
  },
  data() {
    return {
      // 当前选项数据
      currentOptions: [],
      // 正在调用loader
      loading: false
    }
  },
  props: {
    name: String,
    label: String,
    itemProps: Object,
    // 选项数据，数据优先顺序，options 
    options: [Array,Object],
    // 加载数据函数,必须返回Promise
    loader: Function,
    // 选项数据对象属性名称映射
    keyMap: {
      type: Object,
      default() {
        return {
          id: 'id',
          label: 'label',
          value: 'value',
          disabled: 'disabled',
          parentId: 'parentId'
        }
      }
    },
  },
  computed: {
    // 如果有name参数，并且是MForm的子组件，即与MForm的model作双向绑定
    // 否则与组件自身的value作双向绑定
    fieldValue: {
      get() {
        if (this.name && this.mForm) {
          const { currentModel } = this.mForm;
          return _get(currentModel, this.name, this.getDefaultValue());
        } else {
          return this.value || this.getDefaultValue();
        }
      },
      set(val) {
        if (this.name && this.mForm) {
          const { currentModel } = this.mForm;
          const model = cloneDeep(currentModel);
          _set(model, this.name, val);
          if (!isEqual(currentModel, model)) {
            this.mForm.currentModel[this.name] = model[this.name];
            this.mForm.currentModel = model;
          }
          //if(this.mForm.triggerChange){
            //this.mForm.triggerChange(model)
          //}
        } else {
          this.$emit("input", val);
        }
      }
    },
    // 输入框组件参数
    innerProps() {
      return {
        name: this.name,
        ...this.$attrs
      };
    }
  },
  watch: {
    // options 为了提高性能，不设置deep
    options: {
      immediate: true,
      handler(val) {
        this.currentOptions = this.initOptions(val)
        // options改变后，会触发表单验证，这里需要清楚验证错误信息
        this.$nextTick(() => {
          this.clearValidate()
        })
      }
    }
  },
  methods: {
    initOptions(val){
      if(isObject(val)){
        const _this = this
        let _ops = [];
        Object.keys(val).forEach(function(key){
          const _itm = {}
          _itm[_this.keyMap.value] = key;
          _itm[_this.keyMap.label] = val[key];
          _ops.push(_itm)
        });
        return  _ops || []
      }else{
        return cloneDeep(val) || []
      }
    },
    // 获取表单项的默认值，不同组件有不同的默认值，可在具体的组件重写这个函数
    getDefaultValue() {
      return "";
    },
    // 清除验证错误信息
    clearValidate() {
      this.$refs.mItem && this.$refs.mItem.clearValidate()
    },
    // 重置字段
    resetField() {
      this.$refs.mItem && this.$refs.mItem.resetField();
    },
    // 传递给输入组件的插槽
    createSlots(slots = []) {
      return slots.map(name => {
        return <template slot={name}>{this.$slots[name]}</template>;
      });
    },
    // 渲染输入组件
    renderComponent(vnode) {
      // 如果组件不是MForm的子组件，不需要包裹Item组件
      if (!this.mForm) {
        return vnode;
      }

      return (
        <el-form-item
          ref="mItem"
          {...{ props: this.itemProps }}
          // el-form-item 的prop用name代替
          prop={this.name}
          label={this.label}
        >
          {// label 插槽
          this.$slots.label ? (
            <template slot="label">{this.$slots.label}</template>
          ) : null}
          {this.$slots.before}
          {vnode}
          {this.$slots.after}
        </el-form-item>
      );
    },
    // 继承输入组件暴露的方法
    extendMethods(ref, names = []) {
      if (!ref) return;

      names.forEach(name => {
        // 子组件的方法加到实例
        this[name] = (...args) => {
          ref[name].apply(ref, args);
        };
      });
    },
    
    // 加载选项数据
    loadOptions() {
      // 数据优先顺序，options > loader 
      if (this.options) {
        this.currentOptions = this.initOptions(this.options)
        return
      }

      if (this.loader) {
        this.loading = true
        this.loader().then(res => {
          this.currentOptions = this.initOptions(res)
        }).finally(() => {
          this.loading = false
        })
        return
      }
      return
    },
  },
  created() {
    this.loadOptions()
  }
};
