const _cloneDeep = require("lodash/cloneDeep");
/**
 * 列表转换树结构
 * @param {Object[]} list 列表数据，约定字段名称： id/parentId/children
 * @param {*} [parentId=null] 父节点的值
 * @param {string} [idKey=id] id字段名称
 * @param {string} [parentIdKey=parentId] parentId字段名称
 * @param {boolean} [withRoot=false] 有根节点id时是否保留返回根节点
 * @return {Array}
 */
export function listToTree(
  list = [],
  parentId = null,
  idKey = "id",
  parentIdKey = "parentId",
  childrenKey = "children",
  withRoot = false
) {
  list = _cloneDeep(list);
  const temp = new Map();
  let tree = [];
  list.forEach(item => {
    temp.set(item[idKey], item);
  });
  for (const item of list) {
    if (item[parentIdKey] === parentId) {
      tree.push(item);
    } else {
      const parent = temp.get(item[parentIdKey]);
      if (parent) {
        if (!parent[childrenKey]) {
          parent[childrenKey] = [];
        }
        parent[childrenKey].push(item);
      }else{
        // 如果指定了父节点，就必须按指定的父节点显示，否则就不显示
        if(!parentId){
          tree.push(item)
        }
      }
    }
  }
  if (parentId && withRoot) {
    const target = list.find(item => {
      return item[idKey] === parentId;
    });
    target[childrenKey] = tree;
    return [target];
  } else {
    return tree;
  }
}

/**
 * 树结构数据相辅助函数
 * @module $ui/utils/tree
 */

/**
 * 遍历树数据节点，查找符合条件的节点
 * @param {Array|Object} tree 数据树，如 {id:1, children:[{id:2}]}
 * @param {Boolean} isFindOne 是否只找最先符合条件的一个
 * @param {Function} fn 查找回调函数，回调参数：item 节点，index节点当前兄弟节点中的索引，data 查找的数据树，函数返回true表示符合条件
 * @param {string} [childrenKey=children] 子级字段名称
 * @returns {Array|Object} 查找结果，isFindOne为true时返回Object， false时返回Array
 */
export function find(tree = [], isFindOne, fn, childrenKey = "children") {
  let result = [];
  tree = Array.isArray(tree) ? tree : [tree];
  for (let i = 0, len = tree.length; i < len; i++) {
    const item = tree[i],
      checked = fn(item, i, tree),
      children = item[childrenKey];
    if (checked) {
      result.push(item);
      if (isFindOne) break;
    }
    if (children) {
      const child = find(children, isFindOne, fn, childrenKey);
      if (child) result = result.concat(child);
    }
  }
  return isFindOne ? result[0] || null : result;
}

/**
 * 查找节点在树结构数组的路径
 * @param {Array|Object} tree 树数据数组， 如 {id:1, children:[{id:2}]}
 * @param {Function} fn 查找回调函数，回调参数：item 节点，index节点当前兄弟节点中的索引，data 查找的数据树，函数返回true表示符合条件
 * @param {string} [childrenKey=children] 子级字段名称
 * @return {Array} 节点路径数组
 */
export function findPath(tree, fn, childrenKey = "children") {
  const path = [];

  function find(array, parent) {
    parent && path.push(parent);
    for (let i = 0, len = array.length; i < len; i++) {
      const item = array[i],
        checked = fn(item, i, array),
        children = item[childrenKey];
      // 找到，记录路径，退出循环
      if (checked) {
        path.push(item);
        return true;
      }
      if (children && children.length > 0) {
        // 在子级找到，退出循环，自己没有，删除记录的父级
        if (find(children, item)) {
          return true;
        } else {
          path.pop();
        }
      }
    }
  }

  find([].concat(tree));
  return path;
}

const _flat = function(
  tree,
  parentNode = null,
  map = {},
  idKey = "id",
  parentIdKey = "parentId",
  childrenKey = "children"
) {
  tree.forEach(item => {
    if (parentNode) {
      item[parentIdKey] = parentNode[idKey];
      delete parentNode[childrenKey]
    }
    map[item[idKey]] = item;
    if (item[childrenKey]) {
      _flat(item.children, item, map, idKey, parentIdKey, childrenKey);
    }
  });
};
/**
 * 将树状数据摊平为一维数组
 * @param {Array} tree 树数据数组， 如 [{id:1, children:[{id:2}]}]
 * @param {string} [idKey=id] 节点唯一编号字段名称
 * @param {string} [parentIdKey=parentId] parentId字段名称
 * @param {string} [childrenKey=children] 子级字段名称
 * @return {Array} 节点路径数组
 */
export function treeToList(
  tree,
  idKey = "id",
  parentIdKey = "parentId",
  childrenKey = "children"
) {
  const map = {};
  tree = _cloneDeep(tree);
  _flat(tree, null, map, idKey, parentIdKey, childrenKey);
  return Object.values(map);
}
